'''Integrate XPI extensions (such as Firefox translations)'''

# this is part of langpack-o-matic, by Martin Pitt <martin.pitt@canonical.com>
# (C) 2011 Canonical Ltd.

import zipfile
import os, os
import shutil
from glob import glob
import xml.dom.minidom
from xml.parsers.expat import ExpatError
import tarfile
import zipfile
import tempfile
import logging

xpi_data_root = 'po2xpi/data'

def get_xpi_id(xpi):
    '''Return ID of given XPI.'''

    try:
        dom_doc = xml.dom.minidom.parseString(zipfile.ZipFile(xpi).open('install.rdf').read())
    except ExpatError as e:
        logging.error('Invalid XPI %s: %s', xpi, str(e))
        return None

    try:
        attr = dom_doc.getElementsByTagName('RDF:Description')[0].attributes['em:id']
    except IndexError:
        attr = dom_doc.getElementsByTagName('Description')[0].attributes['em:id']

    assert attr.value
    return attr.value

def _make_tar_from_xpi(xpi, output_tar, id):
    '''Unpack an XPI into a mozilla.tar.gz suitable for Firefox < 4.0 langpacks'''

    workdir = tempfile.mkdtemp()
    dest = os.path.join(workdir, 'usr/lib/firefox-addons/extensions', id)
    try:
        zip = zipfile.ZipFile(xpi)
        zip.extractall(dest)
        zip.close()
        if os.path.exists(output_tar):
            tar = tarfile.open(output_tar) 
            tar.extractall(workdir)
            tar.close()
        tar = tarfile.open(output_tar, 'w:gz') 
        tar.add(workdir, '.')
        tar.close()
    finally:
        shutil.rmtree(workdir)

def install_xpis(release_version, locale, source_pkg):
    '''Install XPIs for given locale into source package tree'''

    # Chinese langpacks are split, treat specially
    if locale.startswith('zh_'):
        lang = locale.split('.')[0].split('@')[0].replace('_', '-')
    else:
        lang = locale.split('_')[0].split('@')[0]

    xpis = glob(os.path.join(xpi_data_root, '%s/*/xpi/%s.xpi' % (release_version, lang))) + \
           glob(os.path.join(xpi_data_root, '%s/*/xpi/%s-*.xpi' % (release_version, lang)))

    destdir = os.path.join(source_pkg, 'data')
    for xpi in xpis:
        id = get_xpi_id(xpi)
        if not os.path.isdir(destdir):
            os.mkdir(destdir)

        if float(release_version) > 11:
            # Firefox 4.0 (in Ubuntu 11.04+) can use packed XPIs
            logging.debug('Copying %s (ID: %s) into %s', xpi, id, source_pkg)
            shutil.copy(xpi, os.path.join(destdir, id + '.xpi'))
        else:
            # older releases need the unpacked XPI; ship a readymade tarball in
            # the langpacks, which debian/rules will unpack
            logging.debug('Unpacking %s (ID: %s) into %s', xpi, id, source_pkg)
            _make_tar_from_xpi(xpi, os.path.join(destdir, 'mozilla.tar.gz'), id)

    # Add Firefox search plugins
    if float(release_version) > 10.09:
        search_plugins = glob(os.path.join('extra-files', 'searchplugins', release_version, lang)) + \
                         glob(os.path.join('extra-files', 'searchplugins', release_version, '%s-*' % lang))
        if search_plugins:
            output_tar = os.path.join(destdir, 'searchplugins.tar')
            logging.debug('Creating Firefox search plugins pack %s', output_tar)
            tar = tarfile.open(output_tar, 'w')
            for lang in search_plugins:
                tar.add(lang, os.path.join('usr/lib/firefox-addons/searchplugins',
                            os.path.basename(lang)))
            tar.close()

if __name__ == '__main__':
    import tempfile, unittest

    class _T(unittest.TestCase):
        def setUp(self):
            self.all_xpis = glob(os.path.join(xpi_data_root, '*/*/xpi/*.xpi'))
            self.assertNotEqual(len(self.all_xpis), 0)

            self.workdir = tempfile.mkdtemp()

        def tearDown(self):
            shutil.rmtree(self.workdir)

        def test_xpi_id(self):
            '''get_xpi_id()'''

            for xpi in self.all_xpis:
                id = get_xpi_id(xpi)
                if id:
                    self.assertTrue(id.endswith('.mozilla.org') or id.endswith('.ubuntu.com'))
                    self.assertTrue(os.path.basename(xpi).split('.')[0].split('-')[0] in id)

        def test_install_xpis_unavailable(self):
            '''install_xpis() for unavailable langpack'''

            # valid release, unknown locale
            install_xpis('11.04', 'zz', self.workdir)
            installed = os.listdir(os.path.join(self.workdir))
            self.assertEqual(installed, [])

            # invalid release, known locale
            install_xpis('10.03', 'de', self.workdir)
            installed = os.listdir(os.path.join(self.workdir))
            self.assertEqual(installed, [])

        def test_install_xpis_nocountry(self):
            '''install_xpis() for language without country'''

            install_xpis('11.04', 'de_DE.UTF-8', self.workdir)
            installed = os.listdir(os.path.join(self.workdir, 'data'))
            self.assertEqual(len(installed), 1, 'installed: ' + ' '.join(installed))
            self.assertTrue('-de@' in installed[0])

        def test_install_xpis_unpacked(self):
            '''install_xpis() for Firefox < 4.0 (unpacked XPI)'''

            install_xpis('10.04', 'es_ES.UTF-8', self.workdir)
            installed = os.listdir(os.path.join(self.workdir, 'data'))
            self.assertEqual(len(installed), 1, 'installed: ' + ' '.join(installed))
            self.assertEqual(installed[0], 'mozilla.tar.gz')
            tar = tarfile.open(os.path.join(self.workdir, 'data',
                'mozilla.tar.gz'))
            names = [n.startswith('./') and n[2:] or n for n in tar.getnames()]
            tar.close()
            self.assertTrue('usr/lib/firefox-addons/extensions/langpack-es-ES@firefox.mozilla.org/chrome/es-ES.jar'
                    in names)
            self.assertTrue('usr/lib/firefox-addons/extensions/langpack-es-AR@firefox.mozilla.org/chrome/es-AR.jar'
                    in names)

        def test_install_xpis_country(self):
            '''install_xpis() for country specific language'''

            install_xpis('11.04', 'es_AR.UTF-8', self.workdir)
            installed = os.listdir(os.path.join(self.workdir, 'data'))
            self.assertTrue(len(installed) > 1, 'installed: ' + ' '.join(installed))
            searching = ['es-AR@', 'es-ES@']
            for i in installed:
                for s in list(searching):
                    if s in i:
                        searching.remove(s)
            self.assertEqual(searching, [], 'should have installed ' + ' '.join(searching))

        def test_install_xpis_zh_cn(self):
            '''install_xpis() for Chinese split (CN)'''

            install_xpis('11.04', 'zh_CN.UTF-8', self.workdir)
            installed = os.listdir(os.path.join(self.workdir, 'data'))
            self.assertEqual(len(installed), 1, 'installed: ' + ' '.join(installed))
            self.assertTrue('-zh-CN@' in installed[0])

        def test_install_xpis_zh_tw(self):
            '''install_xpis() for Chinese split (TW)'''

            install_xpis('11.04', 'zh_TW.UTF-8', self.workdir)
            installed = os.listdir(os.path.join(self.workdir, 'data'))
            self.assertEqual(len(installed), 1, 'installed: ' + ' '.join(installed))
            self.assertTrue('-zh-TW@' in installed[0])

    unittest.main()
